#include "mvColumns.h"
#include "mvPythonTranslator.h"
#include "mvGlobalIntepreterLock.h"
#include "mvApp.h"

namespace Marvel {

	void mvManagedColumns::InsertParser(std::map<std::string, mvPythonParser>* parsers)
	{
		parsers->insert({ "add_managed_columns", mvPythonParser({
			{mvPythonDataType::String, "name"},
			{mvPythonDataType::Integer, "columns"},
			{mvPythonDataType::KeywordOnly},
			{mvPythonDataType::Bool, "border", "show border", "True"},
			{mvPythonDataType::Bool, "show", "Attempt to render", "True"},
			{mvPythonDataType::String, "parent", "Parent this item will be added to. (runtime adding)", "''"},
			{mvPythonDataType::String, "before", "This item will be displayed before the specified item in the parent. (runtime adding)", "''"},
		}, "Adds managed columns.", "None", "Containers") });
	}

	void mvColumn::InsertParser(std::map<std::string, mvPythonParser>* parsers)
	{
		parsers->insert({ "add_columns", mvPythonParser({
			{mvPythonDataType::String, "name"},
			{mvPythonDataType::Integer, "columns"},
			{mvPythonDataType::KeywordOnly},
			{mvPythonDataType::Bool, "border", "show border", "True"},
			{mvPythonDataType::Bool, "show", "Attempt to render", "True"},
			{mvPythonDataType::String, "parent", "Parent this item will be added to. (runtime adding)", "''"},
			{mvPythonDataType::String, "before", "This item will be displayed before the specified item in the parent. (runtime adding)", "''"},
		}, "Sets columns.", "None", "Containers") });
	}

	void mvNextColumn::InsertParser(std::map<std::string, mvPythonParser>* parsers)
	{
		parsers->insert({ "add_next_column", mvPythonParser({
			{mvPythonDataType::KeywordOnly},
			{mvPythonDataType::String, "name", "", "'next_collumn'"},
			{mvPythonDataType::Bool, "show", "Attempt to render", "True"},
			{mvPythonDataType::String, "parent", "Parent this item will be added to. (runtime adding)", "''"},
			{mvPythonDataType::String, "before", "This item will be displayed before the specified item in the parent. (runtime adding)", "''"},
		}, "Changes to next column.", "None", "Containers") });
	}

	mvManagedColumns::mvManagedColumns(const std::string& name, int columns)
		: mvAppItem(name)
	{
		if (columns < 1)
			m_columns = 1;
		else if (columns > 64)
			m_columns = 64;
		else
			m_columns = columns;

		for (int i = 0; i < m_columns; i++)
			m_widths.push_back(0);
		m_description.container = true;
	}

	void mvManagedColumns::setColumnWidth(int i, float width)
	{
		if (i < 0 || i >= (int)m_widths.size())
			return;

		m_dirty_widths = true;

		m_widths[i] = width;
	}

	float mvManagedColumns::getColumnWidth(int i)
	{
		if (i < 0 || i >= (int)m_widths.size())
			return 0.0f;

		return m_widths[i];
	}

	void mvManagedColumns::draw()
	{
		m_previousColCount = ImGui::GetColumnsCount();
		ScopedID id;

		ImGui::Columns(m_columns, m_name.c_str(), m_border);
		for (auto& item : m_children)
		{
			// skip item if it's not shown
			if (!item->m_show)
				continue;

			// set item width
			if (item->m_width != 0)
				ImGui::SetNextItemWidth((float)item->m_width);

			//item->pushColorStyles();
			item->draw();
			//item->popColorStyles();

			// Regular Tooltip (simple)
			if (!item->m_tip.empty() && ImGui::IsItemHovered())
				ImGui::SetTooltip("%s", item->m_tip.c_str());

			item->getState().update();

			int index = ImGui::GetColumnIndex();

			if (m_dirty_widths && !m_firstFrame)
				ImGui::SetColumnWidth(index, m_widths[index]);

			if (!m_firstFrame)
				m_widths[index] = ImGui::GetColumnWidth();
			ImGui::NextColumn();
		}

		// this is odd but is necessary for column widths to 
		// work properly
		if (m_firstFrame)
			m_firstFrame = false;
		else
			m_dirty_widths = false;

		ImGui::Columns(m_previousColCount);

	}

	void mvManagedColumns::setExtraConfigDict(PyObject* dict)
	{
		if (dict == nullptr)
			return;
		 
		if (PyObject* item = PyDict_GetItemString(dict, "border")) m_border = ToBool(item);
		if (PyObject* item = PyDict_GetItemString(dict, "columns"))
		{
			m_columns = ToInt(item);

			if (m_columns < 1)
				m_columns = 1;
			else if (m_columns > 64)
				m_columns = 64;

			m_widths.clear();
			for (int i = 0; i < m_columns; i++)
				m_widths.push_back(0);
		}
	}

	void mvManagedColumns::getExtraConfigDict(PyObject* dict)
	{
		if (dict == nullptr)
			return;
		 
		PyDict_SetItemString(dict, "border", ToPyBool(m_border));
		PyDict_SetItemString(dict, "columns", ToPyInt(m_columns));
	}

	mvColumn::mvColumn(const std::string& name, int columns)
		: mvAppItem(name)
	{
		if (columns < 1)
			m_columns = 1;
		else if (columns > 64)
			m_columns = 64;
		else
			m_columns = columns;
	}

	void mvColumn::draw()
	{
		ScopedID id;
		ImGui::Columns(m_columns, m_name.c_str(), m_border);
	}

	void mvColumn::setExtraConfigDict(PyObject* dict)
	{
		if (dict == nullptr)
			return;
		 
		if (PyObject* item = PyDict_GetItemString(dict, "border")) m_border = ToBool(item);
		if (PyObject* item = PyDict_GetItemString(dict, "columns"))
		{
			m_columns = ToInt(item);

			if (m_columns < 1)
				m_columns = 1;
			else if (m_columns > 64)
				m_columns = 64;
		}
	}

	void mvColumn::getExtraConfigDict(PyObject* dict)
	{
		if (dict == nullptr)
			return;
		 
		PyDict_SetItemString(dict, "border", ToPyBool(m_border));
		PyDict_SetItemString(dict, "columns", ToPyInt(m_columns));
	}

	mvNextColumn::mvNextColumn(const std::string& name)
		: mvAppItem(name)
	{
	}

	void mvNextColumn::draw()
	{
		ImGui::NextColumn();
	}

	PyObject* add_managed_columns(PyObject* self, PyObject* args, PyObject* kwargs)
	{
		const char* name;
		int columns;
		int border = true;
		int show = true;
		const char* parent = "";
		const char* before = "";


		if (!(*mvApp::GetApp()->getParsers())["add_managed_columns"].parse(args, kwargs, __FUNCTION__,
			&name, &columns, &border, &show, &parent, &before))
			return ToPyBool(false);

		auto item = CreateRef<mvManagedColumns>(name, columns);
		item->checkConfigDict(kwargs);
		item->setConfigDict(kwargs);
		item->setExtraConfigDict(kwargs);

		if (mvApp::GetApp()->getItemRegistry().addItemWithRuntimeChecks(item, parent, before))
		{
			mvApp::GetApp()->getItemRegistry().pushParent(item);
			if (!show)
				item->hide();

		}

		return GetPyNone();
	}

	PyObject* add_next_column(PyObject* self, PyObject* args, PyObject* kwargs)
	{
		static int i = 0; i++;
		std::string sname = std::string("next_column" + std::to_string(i));
		const char* name = sname.c_str();
		const char* before = "";
		const char* parent = "";
		int show = true;

		if (!(*mvApp::GetApp()->getParsers())["add_next_column"].parse(args, kwargs, __FUNCTION__,
			&name, &show, &parent, &before))
			return ToPyBool(false);

		auto item = CreateRef<mvNextColumn>(name);
		item->checkConfigDict(kwargs);
		item->setConfigDict(kwargs);
		item->setExtraConfigDict(kwargs);

		mvApp::GetApp()->getItemRegistry().addItemWithRuntimeChecks(item, parent, before);

		return GetPyNone();
	}

	PyObject* add_columns(PyObject* self, PyObject* args, PyObject* kwargs)
	{
		const char* name;
		int columns;
		int border = true;
		int show = true;
		const char* parent = "";
		const char* before = "";


		if (!(*mvApp::GetApp()->getParsers())["add_columns"].parse(args, kwargs, __FUNCTION__,
			&name, &columns, &border, &show, &parent, &before))
			return ToPyBool(false);

		auto item = CreateRef<mvColumn>(name, columns);
		item->checkConfigDict(kwargs);
		item->setConfigDict(kwargs);
		item->setExtraConfigDict(kwargs);

		mvApp::GetApp()->getItemRegistry().addItemWithRuntimeChecks(item, parent, before);

		return GetPyNone();
	}
}